@@ -1,17 +1,17 @@
 import logging
-import time
 from os import makedirs, path
 from pathlib import Path
 from typing import Tuple
 
 import wkw
 
+from ..utils import time_start, time_stop
+
 
 def compress_file_job(args: Tuple[Path, Path]) -> None:
     source_path, target_path = args
     try:
-        logging.debug("Compressing '{}' to '{}'".format(source_path, target_path))
-        ref_time = time.time()
+        time_start("Compressing '{}' to '{}'".format(source_path, target_path))
 
         makedirs(path.dirname(target_path), exist_ok=True)
         wkw.File.compress(str(source_path), str(target_path))
@@ -19,11 +19,12 @@ def compress_file_job(args: Tuple[Path, Path]) -> None:
         if not path.exists(target_path):
             raise Exception("Did not create compressed file {}".format(target_path))
 
-        logging.debug(
-            "Compressing of '{}' took {:.8f}s".format(
-                source_path, time.time() - ref_time
+        time_stop("Compressing '{}' to '{}'".format(source_path, target_path))
+
+    except Exception as exc:
+        logging.error(
+            "Compression of '{}' to '{}' failed with {}".format(
+                source_path, target_path, exc
             )
         )
-    except Exception as exc:
-        logging.error("Compressing of '{}' failed with {}".format(source_path, exc))
         raise exc
