@@ -1,5 +1,4 @@
 import logging
-import math
 import operator
 import os
 import re
@@ -405,7 +404,7 @@ class Layer:
 
             foreign_mag_view_path = Path(foreign_mag_view_or_path)
             foreign_mag_view = (
-                Dataset(foreign_mag_view_path.parent.parent)
+                Dataset.open(foreign_mag_view_path.parent.parent)
                 .get_layer(foreign_mag_view_path.parent.name)
                 .get_mag(foreign_mag_view_path.name)
             )
@@ -415,7 +414,7 @@ class Layer:
         foreign_normalized_mag_path = (
             Path(os.path.relpath(foreign_mag_view.path, self.path))
             if make_relative
-            else Path(os.path.abspath(foreign_mag_view.path))
+            else foreign_mag_view.path.resolve()
         )
 
         if symlink:
@@ -501,11 +500,6 @@ class Layer:
         Updates the offset and size of the bounding box of this layer in the properties.
         """
         self._properties.bounding_box = bbox
-
-        for mag, mag_view in self.mags.items():
-            bbox_in_mag = bbox.align_with_mag(mag, ceil=True).in_mag(mag)
-            mag_view._size = bbox_in_mag.bottomright
-            # The offset is not updated since a MagView always is in global coordinates.
         self.dataset._export_as_json()
 
     def downsample(
@@ -558,6 +552,13 @@ class Layer:
         if max_mag is None:
             max_mag = calculate_default_max_mag(self.bounding_box.size)
 
+        if self._properties.bounding_box.size.z == 1:
+            if sampling_mode != SamplingModes.CONSTANT_Z:
+                warnings.warn(
+                    "The sampling_mode was changed to 'CONSTANT_Z'. Downsampling 2D data with a different sampling mode mixes in black and thus leads to darkened images."
+                )
+                sampling_mode = SamplingModes.CONSTANT_Z
+
         scale: Optional[Tuple[float, float, float]] = None
         if sampling_mode == SamplingModes.ANISOTROPIC or sampling_mode == "auto":
             scale = self.dataset.scale
@@ -638,9 +639,7 @@ class Layer:
 
         prev_mag_view = self.mags[from_mag]
 
-        mag_factors = [
-            t // s for (t, s) in zip(target_mag.to_list(), from_mag.to_list())
-        ]
+        mag_factors = target_mag.to_vec3_int() // from_mag.to_vec3_int()
 
         if target_mag in self.mags.keys() and allow_overwrite:
             target_mag_view = self.get_mag(target_mag)
@@ -673,11 +672,6 @@ class Layer:
 
         # perform downsampling
         with get_executor_for_args(args) as executor:
-            voxel_count_per_cube = np.prod(prev_mag_view._get_file_dimensions())
-            job_count_per_log = math.ceil(
-                1024 ** 3 / voxel_count_per_cube
-            )  # log every gigavoxel of processed data
-
             if buffer_edge_len is None:
                 buffer_edge_len = determine_buffer_edge_len(
                     prev_mag_view
@@ -687,21 +681,16 @@ class Layer:
                 mag_factors=mag_factors,
                 interpolation_mode=parsed_interpolation_mode,
                 buffer_edge_len=buffer_edge_len,
-                job_count_per_log=job_count_per_log,
             )
 
             source_view.for_zipped_chunks(
                 # this view is restricted to the bounding box specified in the properties
                 func,
                 target_view=target_view,
-                source_chunk_size=np.array(target_mag_view._get_file_dimensions())
-                * mag_factors,
-                target_chunk_size=target_mag_view._get_file_dimensions(),
                 executor=executor,
+                progress_desc=f"Downsampling layer {self.name} from Mag {from_mag} to Mag {target_mag}",
             )
 
-        logging.info("Mag {0} successfully cubed".format(target_mag))
-
     def redownsample(
         self,
         interpolation_mode: str = "default",
@@ -835,10 +824,6 @@ class Layer:
 
             # perform upsampling
             with get_executor_for_args(args) as executor:
-                voxel_count_per_cube = np.prod(prev_mag_view._get_file_dimensions())
-                job_count_per_log = math.ceil(
-                    1024 ** 3 / voxel_count_per_cube
-                )  # log every gigavoxel of processed data
 
                 if buffer_edge_len is None:
                     buffer_edge_len = determine_buffer_edge_len(
@@ -848,20 +833,15 @@ class Layer:
                     upsample_cube_job,
                     mag_factors=mag_factors,
                     buffer_edge_len=buffer_edge_len,
-                    job_count_per_log=job_count_per_log,
                 )
                 prev_mag_view.get_view().for_zipped_chunks(
                     # this view is restricted to the bounding box specified in the properties
                     func,
                     target_view=target_view,
-                    source_chunk_size=target_mag_view._get_file_dimensions(),
-                    target_chunk_size=target_mag_view._get_file_dimensions()
-                    * np.array([int(1 / f) for f in mag_factors]),
                     executor=executor,
+                    progress_desc=f"Upsampling from Mag {prev_mag} to Mag {target_mag}",
                 )
 
-            logging.info("Mag {0} successfully cubed".format(target_mag))
-
     def _setup_mag(self, mag: Mag) -> None:
         # This method is used to initialize the mag when opening the Dataset. This does not create e.g. the wk_header.
 
