@@ -77,6 +77,12 @@ class Vec3Int(tuple):
     def contains(self, needle: int) -> bool:
         return self.x == needle or self.y == needle or self.z == needle
 
+    def is_positive(self, strictly_positive: bool = False) -> bool:
+        if strictly_positive:
+            return all(i > 0 for i in self)
+        else:
+            return all(i >= 0 for i in self)
+
     def _element_wise(
         self, other: Union[int, "Vec3IntLike"], fn: Callable[[int, Any], int]
     ) -> "Vec3Int":
@@ -124,6 +130,9 @@ class Vec3Int(tuple):
     def __repr__(self) -> str:
         return f"Vec3Int({self.x},{self.y},{self.z})"
 
+    def add_or_none(self, other: Optional["Vec3Int"]) -> Optional["Vec3Int"]:
+        return None if other is None else self + other
+
     @classmethod
     def zeros(cls) -> "Vec3Int":
         return cls(0, 0, 0)
@@ -137,6 +146,4 @@ class Vec3Int(tuple):
         return cls(an_int, an_int, an_int)
 
 
-Vec3IntLike = Union[
-    Vec3Int, Tuple[int, int, int], Tuple[int, ...], np.ndarray, List[int], Iterable[int]
-]
+Vec3IntLike = Union[Vec3Int, Tuple[int, int, int], np.ndarray, Iterable[int]]
