@@ -12,14 +12,17 @@ from multiprocessing import cpu_count
 from pathlib import Path
 from typing import Any, Callable, Iterable, List, Optional, Union
 
+import rich
 from cluster_tools import WrappedProcessPoolExecutor, get_executor
 from cluster_tools.schedulers.cluster_executor import ClusterExecutor
+from rich.progress import Progress
 
 times = {}
 
 
 def time_start(identifier: str) -> None:
     times[identifier] = time.time()
+    logging.debug("{} started".format(identifier))
 
 
 def time_stop(identifier: str) -> None:
@@ -84,12 +87,22 @@ def named_partial(func: F, *args: Any, **kwargs: Any) -> F:
     return partial_func
 
 
-def wait_and_ensure_success(futures: List[Future]) -> List[Any]:
+def wait_and_ensure_success(
+    futures: List[Future], progress_desc: Optional[str] = None
+) -> List[Any]:
     """Waits for all futures to complete and raises an exception
     as soon as a future resolves with an error."""
+
     results = []
-    for fut in as_completed(futures):
-        results.append(fut.result())
+    if progress_desc is None:
+        for fut in as_completed(futures):
+            results.append(fut.result())
+    else:
+        with get_rich_progress() as progress:
+            task = progress.add_task(progress_desc, total=len(futures))
+            for fut in as_completed(futures):
+                results.append(fut.result())
+                progress.update(task, advance=1)
     return results
 
 
@@ -122,9 +135,9 @@ def copy_directory_with_symlinks(
         if item.name not in ignore:
             symlink_path = dst_path / item.name
             if make_relative:
-                rel_or_abspath = os.path.relpath(item, symlink_path.parent)
+                rel_or_abspath = Path(os.path.relpath(item, symlink_path.parent))
             else:
-                rel_or_abspath = os.path.abspath(item)
+                rel_or_abspath = item.resolve()
             symlink_path.symlink_to(rel_or_abspath)
 
 
@@ -137,7 +150,18 @@ def setup_logging(args: argparse.Namespace) -> None:
 
 def add_verbose_flag(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
-        "--silent", help="Silent output", dest="verbose", action="store_false"
+        "--verbose", help="Verbose output", dest="verbose", action="store_true"
     )
 
-    parser.set_defaults(verbose=True)
+    parser.set_defaults(verbose=False)
+
+
+def get_rich_progress() -> Progress:
+    return Progress(
+        "[progress.description]{task.description}",
+        rich.progress.BarColumn(),
+        "[progress.percentage]{task.percentage:>3.0f}%",
+        rich.progress.TimeElapsedColumn(),
+        "|",
+        rich.progress.TimeRemainingColumn(),
+    )
